@@ -11,10 +11,10 @@ from __future__ import annotations
 
 import binascii
 import re
-from contextlib import suppress
 from enum import Enum
 from typing import TYPE_CHECKING, cast
 
+import svg_path_data
 from lxml import etree
 
 from svg_ultralight.nsmap import NSMAP
@@ -27,109 +27,16 @@ if TYPE_CHECKING:
     )
 
 
-_MAYBE_NEG = r"(?:(?P<negative>-?))"
-_MAYBE_INT = r"(?:(?P<integer>\d+?))"
-_MAYBE_FRACTION = r"(?:\.(?P<fraction>\d+))?"
-_MAYBE_EXP = r"(?:[eE](?P<exponent>[+-]?\d+))?"
+def format_number(num: float | str, resolution: int | None = 6) -> str:
+    """Format a number into an svg-readable float string with resolution = 6.
 
-# Split a float (fp or exponential) into its components. All components are optional.
-FLOAT_PATTERN = re.compile(rf"{_MAYBE_NEG}{_MAYBE_INT}{_MAYBE_FRACTION}{_MAYBE_EXP}")
-
-
-def _split_float_str(num: str | float) -> tuple[str, str, str, int]:
-    """Split a float string into its sign, integer part, fractional part, and exponent.
-
-    :param num_str: A string representing the number (e.g., '1.23e+03').
-    :return: A tuple containing the integer part, fractional part, and exponent.
+    :param num: number to format (string or float)
+    :param resolution: number of digits after the decimal point, defaults to 6. None
+        to match behavior of `str(num)`.
+    :return: string representation of the number with six digits after the decimal
+        (if in fixed-point notation). Will return exponential notation when shorter.
     """
-    if float(num) == 0:
-        return "", "", "", 0
-    num_str = str(num)
-    groups = FLOAT_PATTERN.fullmatch(num_str)
-    if not groups:
-        msg = "Invalid number string: {num_str}."
-        raise ValueError(msg.format(num_str=num_str))
-
-    sign = groups["negative"] or ""
-    integer = (groups["integer"] or "").lstrip("0")
-    fraction = (groups["fraction"] or "").rstrip("0")
-    exponent = int(groups["exponent"] or 0)
-    return sign, integer, fraction, exponent
-
-
-def _format_as_fixed_point(num: str | float) -> str:
-    """Format a number in fixed-point notation.
-
-    :param exp_str: A string representing the number in exponential notation
-        (e.g., '1.23e+03') or just a number.
-    :return: A string representing the number in fixed-point notation.
-    """
-    sign, integer, fraction, exponent = _split_float_str(num)
-    if exponent > 0:
-        fraction = fraction.ljust(exponent, "0")
-        integer += fraction[:exponent]
-        fraction = fraction[exponent:]
-    elif exponent < 0:
-        integer = integer.rjust(-exponent, "0")
-        fraction = integer[exponent:] + fraction
-        integer = integer[:exponent]
-
-    fraction = "." + fraction if fraction else ""
-    return f"{sign}{integer}{fraction}" or "0"
-
-
-def _format_as_exponential(num: str | float) -> str:
-    """Convert a number in fixed-point notation (as a string) to exponential notation.
-
-    :param num_str: A string representing the number in fixed-point notation
-        (e.g., '123000') or just a number.
-    :return: A string representing the number in exponential notation.
-    """
-    sign, integer, fraction, exponent = _split_float_str(num)
-    if len(integer) > 1:
-        exponent += len(integer) - 1
-        fraction = (integer[1:] + fraction).rstrip("0")
-        integer = integer[0]
-    elif not integer and fraction:
-        leading_zeroes = len(fraction) - len(fraction.lstrip("0"))
-        exponent -= leading_zeroes + 1
-        integer = fraction[leading_zeroes]
-        fraction = fraction[leading_zeroes + 1 :]
-
-    fraction = "." + fraction if fraction else ""
-    exp_str = f"e{exponent}" if exponent else ""
-    return f"{sign}{integer}{fraction}{exp_str}" or "0"
-
-
-def format_number(num: float | str, precision: float | None = 6) -> str:
-    """Format strings at limited precision.
-
-    :param num: anything that can print as a float.
-    :param precision: number of digits after the decimal point, default 6. You can
-        also pass None for no precision limit. This may produce some long strings,
-        but will retain as much information as possible when converting between
-        floats and strings.
-    :return: str
-
-    I've read articles that recommend no more than four digits before and two digits
-    after the decimal point to ensure good svg rendering. I'm being generous and
-    giving six. Mostly to eliminate exponential notation, but I'm "rstripping" the
-    strings to reduce filesize and increase readability
-
-    * reduce fp precision to (default) 6 digits
-    * remove trailing zeros
-    * remove trailing decimal point
-    * remove leading 0 in "0.123"
-    * convert "-0" to "0"
-    * use shorter of exponential or fixed-point notation
-    """
-    if precision is not None:
-        num = f"{float(num):.{precision}f}"
-    exponential_str = _format_as_exponential(num)
-    fixed_point_str = _format_as_fixed_point(num)
-    if len(exponential_str) < len(fixed_point_str):
-        return exponential_str
-    return fixed_point_str
+    return svg_path_data.format_number(num, resolution=resolution)
 
 
 def format_numbers(
@@ -143,44 +50,6 @@ def format_numbers(
     return [format_number(num) for num in nums]
 
 
-def _is_float_or_float_str(data: float | str) -> bool:
-    """Check if a string is a float.
-
-    :param data: string to check
-    :return: bool
-    """
-    try:
-        _ = float(data)
-    except ValueError:
-        return False
-    else:
-        return True
-
-
-def format_numbers_in_string(data: float | str) -> str:
-    """Find and format floats in a string.
-
-    :param data: string with floats or a float value
-    :return: string with floats formatted to limited precision
-
-    Works as a more robust version of format_number. Will correctly handle input
-    floats in exponential notation. This should work for any parameter value in an
-    svg except 'text', 'id' and for any other value except hex color codes. The
-    function will fail with input strings like 'ice3.14bucket', because 'e3.14' will
-    be identified as a float. SVG param values will not have such strings, but the
-    'text' attribute could. This function will not handle that case. Do not attempt
-    to reformat 'text' attribute values.
-    """
-    with suppress(ValueError):
-        # try as a regular number to strip spaces from simple float strings
-        return format_number(data)
-    if str(data).startswith("#"):
-        return str(data)
-    words = re.split(r"([^\d.eE-]+)", str(data))
-    words = [format_number(w) if _is_float_or_float_str(w) else w for w in words]
-    return "".join(words)
-
-
 def _fix_key_and_format_val(key: str, val: str | float) -> tuple[str, str]:
     """Format one key, value pair for an svg element.
 
@@ -213,10 +82,12 @@ def _fix_key_and_format_val(key: str, val: str | float) -> tuple[str, str]:
     else:
         key_ = key.rstrip("_").replace("_", "-")
 
-    if key_ in {"id", "text"}:
-        return key_, str(val)
+    if isinstance(val, (int, float)):
+        val_ = format_number(val)
+    else:
+        val_ = val
 
-    return key_, format_numbers_in_string(val)
+    return key_, val_
 
 
 def format_attr_dict(**attributes: str | float) -> dict[str, str]:
